@@ -10,7 +10,6 @@ import tempfile
 import typing
 from itertools import chain
 from random import choice
-from types import GeneratorType
 
 from .utils import BaseCtx, BaseOptions, Code
 
@@ -87,37 +86,24 @@ class _ConverterCallable:
         return _ConverterCallableMethod(self, instance, cls)
 
     def __call__(self, *args, **kwargs):
-        drop_labels_now = True
         try:
-            result = self._main_converter(*args, **kwargs)
-            if isinstance(result, GeneratorType):
-                drop_labels_now = False
-                return self.wrap_generator_clean_labels_on_exit(result)
-            return result
+            return self._main_converter(*args, **kwargs)
         except (Exception, KeyboardInterrupt):
-            # definitely not generator
             self.dump_code()
             raise
-        finally:
-            if drop_labels_now:
-                labels_ = self._ctx["labels_"]
-                if labels_:
-                    for key in list(labels_):
-                        del labels_[key]
-
-    def wrap_generator_clean_labels_on_exit(self, generator_):
-        try:
-            yield from generator_
-        except Exception:
-            self.dump_code()
-            raise
-        finally:
-            labels_ = self._ctx["labels_"]
-            if labels_:
-                for key in list(labels_):
-                    del labels_[key]
 
     def dump_code(self):
+        """Run this method if you are consuming iterator, returned from the
+        conversion, and you need proper tracebacks:
+
+            >>> converter = c.iter({...}).gen_converter()
+                try:
+                    for item in converter(data):
+                        ...
+                except Exception:
+                    converter.dump_code()
+                    raise
+        """
         if self._code_dumped:
             return
         cls = self.__class__
@@ -180,14 +166,12 @@ class ConverterOptionsCtx(BaseCtx):
 
 CONVERTER_TEMPLATE = """
 def {converter_name}({code_signature}):
-    global labels_
 {code}
 """
 
 
 GET_OR_DEFAULT_TEMPLATE = """
 def {converter_name}({code_args}):
-    global labels_
     try:
         return {get_or_default_code}
     except (TypeError, KeyError, IndexError, AttributeError):
@@ -313,13 +297,11 @@ class BaseConversion(typing.Generic[CT]):
             self.contents |= arg.contents
         return self
 
-    def get_dependencies(self, types=None, exclude_types=None):
+    def get_dependencies(self, types=None):
         deps = self._depends_on.values()
         deps = chain(deps, (self,))
         if types:
             deps = (dep for dep in deps if isinstance(dep, types))
-        if exclude_types:
-            deps = (dep for dep in deps if not isinstance(dep, exclude_types))
         return deps
 
     def ensure_conversion(self, conversion, **kwargs) -> "BaseConversion":
@@ -398,13 +380,11 @@ class BaseConversion(typing.Generic[CT]):
             where,
         )
 
-    def get_args(self, exclude_types=None):
+    def get_args(self):
         return sorted(
             {
                 dep.arg_name: dep
-                for dep in self.get_dependencies(
-                    types=InputArg, exclude_types=exclude_types
-                )
+                for dep in self.get_dependencies(types=InputArg)
             }.values(),
             key=lambda k: k.arg_name,
         )
@@ -412,35 +392,40 @@ class BaseConversion(typing.Generic[CT]):
     def get_args_def_code(
         self,
         as_kwargs=False,
-        exclude_cls_self=False,
-        exclude_labels=True,
+        for_top_level_converter=False,
     ):
         """Generates the code to define a function signature based on InputArgs
         used inside the conversion"""
-        args = self.get_args(
-            exclude_types=(LabelConversion,) if exclude_labels else None
-        )
-
-        if exclude_cls_self:
+        args = self.get_args()
+        if for_top_level_converter:
             args = [arg for arg in args if arg.arg_name not in ("self", "cls")]
-        if not args:
-            return ""
 
-        code = ", ".join(arg.arg_name for arg in args)
-        if as_kwargs:
-            return ", *, {}".format(code)
-        return ", {}".format(code)
+        if args:
+            code = ", ".join(arg.arg_name for arg in args)
+            if as_kwargs:
+                code = f", *, {code}"
+            else:
+                code = f", {code}"
+        else:
+            code = ""
+        if not for_top_level_converter:
+            code = f", _naive, _labels, _none{code}"
+        return code
 
-    def get_args_as_func_args(self, exclude_labels=True):
+    def get_args_as_func_args(self):
         """Generates the code to pass InputArgs as arguments to the function"""
-        args = self.get_args(
-            exclude_types=(LabelConversion,) if exclude_labels else None
-        )
+        args = self.get_args()
         ctx = {}
-        return tuple(
-            EscapedString(arg.gen_code_and_update_ctx(None, ctx))
-            for arg in args
-        )
+        func_args = [
+            EscapedString("_naive"),
+            EscapedString("_labels"),
+            EscapedString("_none"),
+        ]
+        for arg in args:
+            func_args.append(
+                EscapedString(arg.gen_code_and_update_ctx(None, ctx))
+            )
+        return func_args
 
     def _code_to_converter(
         self, converter_name: str, code: str, ctx: dict
@@ -471,13 +456,12 @@ class BaseConversion(typing.Generic[CT]):
 
     @classmethod
     def _init_ctx(cls, debug=None):
-        labels_: typing.Dict[str, typing.Any] = {}
         ctx = {
             "sys": sys,
             "__debug": debug,
             "__name__": "_convtools",
-            "labels_": labels_,
-            "_none": cls._none,
+            "__naive_values__": {},
+            "__none__": cls._none,
             cls.NAME_TO_CODE_INPUT: [{}],
             cls.PREFIXED_HASH_TO_NAME: {},
             cls.GENERATED_NAMES: set(),
@@ -523,11 +507,16 @@ class BaseConversion(typing.Generic[CT]):
 
         if signature:
             signature_words = _pattern_word.findall(signature)
-            missing_args = set(
-                _pattern_word.findall(
-                    self.get_args_def_code(exclude_labels=True)
+            missing_args = (
+                set(
+                    _pattern_word.findall(
+                        self.get_args_def_code(
+                            for_top_level_converter=True,
+                        )
+                    )
                 )
-            ) - set(signature_words)
+                - set(signature_words)
+            )
             if missing_args:
                 raise ConversionException(
                     "bad signature, missing args", missing_args
@@ -544,8 +533,7 @@ class BaseConversion(typing.Generic[CT]):
                 + (
                     self.get_args_def_code(
                         as_kwargs=True,
-                        exclude_cls_self=True,
-                        exclude_labels=True,
+                        for_top_level_converter=True,
                     )
                 )
             )
@@ -553,7 +541,10 @@ class BaseConversion(typing.Generic[CT]):
         code = Code()
         converter_name = self.gen_name(converter_name, ctx, self)
         code.add_line(f"def {converter_name}({signature}):", 1)
-        code.add_line("global labels_", 0)
+        code.add_line("global __naive_values__, __none__", 0)
+        code.add_line("_naive = __naive_values__", 0)
+        code.add_line("_none = __none__", 0)
+        code.add_line("_labels = {}", 0)
         code.add_line(
             "return {}".format(
                 self.gen_code_and_update_ctx(initial_code_input, ctx)
@@ -570,9 +561,11 @@ class BaseConversion(typing.Generic[CT]):
         del ctx[self.GENERATED_NAMES]
         return main_converter_callable
 
-    def execute(self, *args, debug=False, **kwargs) -> typing.Any:
+    def execute(self, *args, debug=None, **kwargs) -> typing.Any:
         """Shortcut for generating converter and running it"""
-        return self.gen_converter(debug=debug)(*args, **kwargs)
+        return self.gen_converter(
+            debug=debug or ConverterOptionsCtx.get_option_value("debug")
+        )(*args, **kwargs)
 
     def iter(self, element_conv, *, where=None) -> "BaseConversion":
         """Shortcut for
@@ -625,6 +618,12 @@ class BaseConversion(typing.Generic[CT]):
         """
         return Call(*args, self_conv=self, **kwargs)
 
+    def apply(self, args, kwargs):
+        """Gets compiled into the code which calls the input with params.
+        Both ``args`` and ``kwargs`` are wrapped with ``ensure_conversion``.
+        """
+        return ApplyFunc(self, args, kwargs)
+
     def call_method(self, method_name: str, *args, **kwargs) -> "Call":
         """Gets compiled into the code which calls the ``method_name`` method
         of input with params.
@@ -632,6 +631,13 @@ class BaseConversion(typing.Generic[CT]):
         """
         return self.attr(method_name).call(*args, **kwargs)
 
+    def apply_method(self, method_name: str, args, kwargs) -> "Call":
+        """Gets compiled into the code which calls the ``method_name`` method
+        of input with params.
+        It's a shortcut to ``(...).attr(method_name).apply(args, kwargs)``
+        """
+        return self.attr(method_name).apply(args, kwargs)
+
     def as_type(self, callable_) -> "Call":
         return ensure_conversion(callable_).call(self)
 
@@ -884,23 +890,21 @@ class NaiveConversion(BaseConversion):
     def _gen_code_and_update_ctx(self, code_input, ctx):
         if self.code_str:
             return self.code_str
-        if self.value is None:
-            return "None"
-        if self.value is True:
-            return "True"
-        if self.value is False:
-            return "False"
+        value = self.value
         if (
-            isinstance(self.value, int)
-            or isinstance(self.value, str)
-            and len(self.value) < 255
+            value is None
+            or isinstance(value, (bool, int))
+            or isinstance(value, str)
+            and len(value) < 255
+            and "%" not in value
+            and "{" not in value
         ):
-            return repr(self.value)
+            return repr(value)
         value_name = self.value_name or self.gen_name(
-            self.name_prefix, ctx, self.value
+            self.name_prefix, ctx, value
         )
-        ctx[value_name] = self.value
-        return value_name
+        ctx["__naive_values__"][value_name] = value
+        return f"_naive['{value_name}']"
 
     def is_itself_callable_like(self) -> typing.Optional[bool]:
         return callable(self.value)
@@ -952,7 +956,7 @@ class InputArg(BaseConversion):
         return self.arg_name
 
 
-class LabelConversion(InputArg):
+class LabelConversion(BaseConversion):
     """Allows to reference a conversion result by label, after it was cached by
     :py:obj:`PipeConversion` or :py:obj:`BaseConversion.add_label`."""
 
@@ -963,11 +967,13 @@ class LabelConversion(InputArg):
         Args:
           label_name (string): label name to be referenced
         """
-        super().__init__(label_name)
-        self.caching_conversion = None
+        super().__init__()
+        if not isinstance(label_name, str):
+            raise ValueError("invalid label_name type")
+        self.label_name = label_name
 
     def _gen_code_and_update_ctx(self, code_input, ctx):
-        return f"labels_['{self.arg_name}']"
+        return f"_labels[{repr(self.label_name)}]"
 
 
 class ConversionWrapper(BaseConversion):
@@ -1277,6 +1283,15 @@ def CallFunc(func, *args, **kwargs) -> "Call":
     return NaiveConversion(func).call(*args, **kwargs)
 
 
+def _applier(func, args, kwargs):
+    return func(*args, **kwargs)
+
+
+def ApplyFunc(func, args, kwargs) -> "Call":
+    """Shortcut to ``NaiveConversion(func).apply(args, kwargs)``"""
+    return NaiveConversion(_applier).call(func, args, kwargs)
+
+
 class FilterConversion(BaseConversion):
     """Generates the code to iterate the input, taking items for which the
     provided conversion resolves to a truth value."""
@@ -1587,7 +1602,6 @@ class BaseCollectionConversion(BaseConversion):
         converter_name = self.gen_name("optional_items_generator", ctx, self)
         code_args = self.get_args_def_code(as_kwargs=False)
         code.add_line(f"def {converter_name}(data_{code_args}):", 1)
-        code.add_line("global labels_", 0)
 
         for condition, item in self.condition_to_item_pairs:
             value_code = ensure_conversion(item).gen_code_and_update_ctx(
@@ -1694,6 +1708,8 @@ class Tuple(BaseCollectionConversion):
     def gen_collection_from_items_code(
         self, joined_items_code, code_input, ctx
     ):
+        if not joined_items_code:
+            return "()"
         return f"({joined_items_code},)"
 
     def gen_collection_from_generator(self, generator_code, code_input, ctx):
@@ -1838,7 +1854,9 @@ class PipeConversion(BaseConversion):
                 "inline pipes requested: no labeling is supported"
             )
 
-        self.what = self.ensure_conversion(what)
+        self.input_args_container = EscapedString("None")
+
+        self.what = self.ensure_conversion(what, skip_for_input_args=True)
         self.where = self.ensure_conversion(where)
         self.where.check_dependency(self.what, for_piping=True)
 
@@ -1850,14 +1868,17 @@ class PipeConversion(BaseConversion):
         if not self.where.valid_pipe_output:
             raise ValueError("invalid output, check where conversion")
 
-        self.replace_where_with_called_one = self.where.is_itself_callable()
-        if not self.replace_where_with_called_one and (args or kwargs):
+        if self.where.is_itself_callable():
+            self.where = self.where.call(
+                GetItem(),
+                *tuple(self.ensure_conversion(arg) for arg in args),
+                **{k: self.ensure_conversion(v) for k, v in kwargs.items()},
+            )
+        elif args or kwargs:
             raise AssertionError(
                 "args or kwargs won't be used when 'where' is not callable"
             )
 
-        self.args = tuple(self.ensure_conversion(arg) for arg in args)
-        self.kwargs = {k: self.ensure_conversion(v) for k, v in kwargs.items()}
         self.label_input = (
             None if label_input is None else self._prepare_labels(label_input)
         )
@@ -1874,10 +1895,8 @@ class PipeConversion(BaseConversion):
         return PipeConversion(
             what=self.what,
             where=where,
-            *self.args,
             label_input=self.label_input,
             label_output=self.label_output,
-            **self.kwargs,
         )
 
     def as_type(self, callable_):
@@ -1889,7 +1908,10 @@ class PipeConversion(BaseConversion):
     def sort(self, key=None, reverse=False):
         return self.replace(self.where.sort(key, reverse))
 
-    def _prepare_labels(self, label_arg: typing.Union[str, dict]):
+    def _prepare_labels(
+        self,
+        label_arg: typing.Union[str, dict],
+    ):
         if isinstance(label_arg, str):
             return {label_arg: GetItem()}
 
@@ -1916,16 +1938,16 @@ class PipeConversion(BaseConversion):
             return code_input.count("][") < 2
         return False
 
+    def ensure_conversion(
+        self, conversion, skip_for_input_args=False, **kwargs
+    ) -> "BaseConversion":
+        if not skip_for_input_args:
+            self.input_args_container.ensure_conversion(conversion)
+        return super().ensure_conversion(conversion, **kwargs)
+
     def _gen_code_and_update_ctx(self, code_input, ctx):
         what_code = self.what.gen_code_and_update_ctx(code_input, ctx)
-        if self.replace_where_with_called_one:
-            where = self.where.call(
-                GetItem(),
-                *self.args,
-                **self.kwargs,
-            )
-        else:
-            where = self.where
+        where = self.where
 
         where_code = where.gen_code_and_update_ctx(what_code, ctx)
         code_usage_count = self.count_words(where_code, what_code)
@@ -1971,13 +1993,13 @@ class PipeConversion(BaseConversion):
             if reducer_inputs_backup_needed:
                 ctx[key_to_backup].pop()
 
-        code_args = where.get_args_def_code()
-        where_args = where.get_args_as_func_args()
+        code_args = self.input_args_container.get_args_def_code()
+        where_args = self.input_args_container.get_args_as_func_args()
 
         if self.label_input or self.label_output:
             label_input_code = (
                 "\n".join(
-                    "    labels_['{var_label}'] = {code_label}".format(
+                    "    _labels['{var_label}'] = {code_label}".format(
                         var_label=label_name,
                         code_label=label_conv.gen_code_and_update_ctx(
                             var_input, ctx
@@ -1990,7 +2012,7 @@ class PipeConversion(BaseConversion):
             )
             label_output_code = (
                 "\n".join(
-                    "    labels_['{var_label}'] = {code_label}".format(
+                    "    _labels['{var_label}'] = {code_label}".format(
                         var_label=label_name,
                         code_label=label_conv.gen_code_and_update_ctx(
                             var_result, ctx
@@ -2003,7 +2025,6 @@ class PipeConversion(BaseConversion):
             )
             code = f"""
 def {converter_name}({var_input}{code_args}):
-    global labels_
 {label_input_code}
     {var_result} = {where_code}
 {label_output_code}
@@ -2012,7 +2033,6 @@ def {converter_name}({var_input}{code_args}):
         else:
             code = f"""
 def {converter_name}({var_input}{code_args}):
-    global labels_
     return {where_code}
         """
 
@@ -2048,9 +2068,7 @@ class TapConversion(BaseConversion):
 
     def _gen_code_and_update_ctx(self, code_input, ctx):
         converter_name = self.gen_name("tap", ctx, self)
-        code_args = self.get_args_def_code(
-            as_kwargs=False, exclude_labels=True
-        )
+        code_args = self.get_args_def_code(as_kwargs=False)
         code = Code()
         code.add_line(f"def {converter_name}(data_{code_args}):", 1)
         for mut in self.mutations:
@@ -2085,9 +2103,7 @@ def {converter_name}(data_{code_args}):
             ctx,
             self,
         )
-        code_args = self.get_args_def_code(
-            as_kwargs=False, exclude_labels=True
-        )
+        code_args = self.get_args_def_code(as_kwargs=False)
         code = Code()
         code.add_line(f"def {converter_name}(data_{code_args}):", 1)
         code.add_line("for item_ in data_:", 1)
