@@ -1,7 +1,7 @@
 """
 This module brings join functionality to the library
 """
-import typing
+import typing as t
 from itertools import chain
 
 from .aggregations import Aggregate, ReduceFuncs
@@ -22,17 +22,37 @@ from .base import (
     NamedConversion,
     Tuple,
 )
+from .columns import ColumnRef
 
 
 class JoinException(Exception):
     pass
 
 
+class JoinCondition(NamedConversion):
+    NAME: str
+
+    def __init__(self):
+        super().__init__(self.NAME, GetItem())
+
+    def col(self, column_name: str) -> BaseConversion:
+
+        return self.pipe(ColumnRef(column_name, id_=self.NAME))
+
+
+class LeftJoinCondition(JoinCondition):
+    NAME = "left_row"
+
+
+class RightJoinCondition(JoinCondition):
+    NAME = "right_row"
+
+
 class _JoinConditions:
     """A helper object to analyze join conditions"""
 
-    LEFT = NamedConversion("left_row", GetItem())
-    RIGHT = NamedConversion("right_row", GetItem())
+    LEFT = LeftJoinCondition()
+    RIGHT = RightJoinCondition()
     LEFT_NAME = LEFT.name
     RIGHT_NAME = RIGHT.name
     _ANY = {LEFT_NAME, RIGHT_NAME}
@@ -82,7 +102,7 @@ class _JoinConditions:
         return join_conditions
 
     @classmethod
-    def _get_join_deps(cls, conv) -> typing.Set[str]:
+    def _get_join_deps(cls, conv) -> t.Set[str]:
         return {
             dep.name
             for dep in conv.get_dependencies(types=NamedConversion)
@@ -166,11 +186,16 @@ class JoinConversion(BaseConversion):
         self.left_conversion = self.ensure_conversion(left_conversion)
         self.right_conversion = self.ensure_conversion(right_conversion)
         self.condition = self.ensure_conversion(condition)
+        self.how = self.validate_how(how)
+
+    @classmethod
+    def validate_how(cls, how: str):
+        how = how.lower()
         if how not in ("inner", "left", "right", "outer", "full"):
             raise ValueError(how)
         if how == "full":
             how = "outer"
-        self.how = how
+        return how
 
     def _gen_code_and_update_ctx(self, code_input, ctx):
         condition = self.condition
@@ -219,7 +244,7 @@ class JoinConversion(BaseConversion):
                     ),
                 )
             )
-        if self.how == "outer":
+        if self.how == "outer" or not left_row_hashers:
             right_collection = right_collection.as_type(list)
         right_collection = right_collection.add_label("right_collection")
 
@@ -240,7 +265,7 @@ class JoinConversion(BaseConversion):
                 )
             )
 
-        inner_loop_condition: typing.Optional[ConversionWrapper] = None
+        inner_loop_condition: t.Optional[ConversionWrapper] = None
         resulting_inner_loop_conditions = list(
             chain(
                 left_row_filters,
@@ -363,7 +388,7 @@ class JoinConversion(BaseConversion):
 
         if self.how == "outer":
 
-            def _add_right_part(left_join_gen, labels_):
+            def _add_right_part(left_join_gen, labels):
                 yielded_right_ids = set()
                 for left_right in left_join_gen:
                     yielded_right_ids.add(id(left_right[1]))
@@ -371,13 +396,13 @@ class JoinConversion(BaseConversion):
 
                 yield from (
                     (None, right)
-                    for right in labels_["right_collection"]
+                    for right in labels["right_collection"]
                     if id(right) not in yielded_right_ids
                 )
 
             conv = conv.pipe(
                 _add_right_part,
-                labels_=EscapedString("labels_"),
+                labels=EscapedString("_labels"),
             )
         if pre_filter:
             conv = If(
@@ -402,7 +427,6 @@ class JoinConversion(BaseConversion):
         code_args = self.get_args_def_code(as_kwargs=False)
         code = f"""
 def {converter_name}(left_, right_{code_args}):
-    global labels_
     return {code_join}
         """
         self._code_to_converter(
